unit HlcFileSearchThread;

interface

uses
  Classes, SysUtils;

type
  TLogFunct = Function( Const path: String; Const SRec: TSearchRec ): Boolean
                of Object;

  THlcFileSearchThread = class(TThread)
  private
    { Private declarations }
    MaskList,
    PathList:TSTringList;
    FRecursive: boolean;
    FPath: string;
    FSRec: TSearchRec;
    FResult: boolean;
    FSearchLogFunction: TLogFunct;
    procedure doSearchLogFunction;
    procedure FindRecursive(const path, mask: String;
      LogFunction: TLogFunct);
  protected
    procedure Execute; override;
    function InternalSearchLogFunction(const path: string; const SRec: TSearchRec): Boolean;
  public
    constructor Create(SearchLogFunction: TLogFunct; Recursive: boolean);
    destructor Destroy; override;
    procedure AddSearchTask(const SearchPath, SearchMask: string);
  end;

implementation

{ THlcFileSearchThread }

constructor THlcFileSearchThread.Create(SearchLogFunction: TLogFunct; Recursive: boolean);
begin
  MaskList := TSTringList.Create;
  PathList := TSTringList.Create;
  FSearchLogFunction := SearchLogFunction;
  FRecursive := Recursive;
  FreeOnTerminate := True;
  inherited Create(True);
end;

procedure THlcFileSearchThread.FindRecursive( const Path: string; const Mask: string;
  LogFunction: TLogFunct );
var
  FullPath: String;

  function Recurse( var Path: string; const Mask: string ): boolean;
  var
    SRec: TSearchRec;
    RetVal: Integer;
    OldLen: Integer;
  begin
    if Terminated then
    begin
      Result := False; {causes outer levels of recursion to end, too}
      Exit;
    end;
    { set default return value: continue scan }
    Recurse := True;
    { remember current length of path so we can chop off added
      subdirectory names again. path is guaranteed to end in a
      backslash here }
    OldLen := Length( path );
    (* phase 1, look for normal files *)
    RetVal := FindFirst( path+mask, faAnyFile, SRec );
    while RetVal = 0 do begin
      if (SRec.Attr and (faDirectory or faVolumeID)) = 0 then
      begin
        (* we found a file, not a directory or volume label,
           log it. Bail out if the log function returns false. *)
        if not LogFunction( path, SRec ) then
        begin
          Result := False; {causes outer levels of recursion to end, too}
          FindClose( SRec );
          Break;
        end;
      end;
      if Terminated then
      begin
        Result := False; {causes outer levels of recursion to end, too}
        FindClose( SRec );
        Break;
      end;
      RetVal := FindNext( SRec );
    end;
    FindClose( SRec );     { added for Win32 compatibility }
    if not Result then Exit;
    if FRecursive then
    begin
      (* Phase II, look for subdirectories and recurse thru them *)
      RetVal := FindFirst( path+'*.*', faDirectory or faSysFile, SRec );
      while RetVal = 0 do begin
        if (SRec.Attr and faDirectory) <> 0 then
          (* we have a directory, but do _not_ recurse thru these
             blasted proxy pseudodirectories standing for the
             current dir and its parent. That would cause an infinit
             recursion loop... *)
          if (SRec.Name <> '.') and (SRec.Name <> '..') then begin
            LogFunction( path, SRec );
            { ok, its a harmless dir, add its name to path and recurse }
            path := path + SRec.Name + '\';
            if not Recurse( path, mask ) then begin
              Result := False;
              FindClose( SRec );
              Break;
            end;
            { remove the added name again so we can stick on the next
              on the next round of the while loop }
            Delete( path, OldLen+1, 255 );
          end;
        if Terminated then
        begin
          Result := False; {causes outer levels of recursion to end, too}
          FindClose( SRec );
          Break;
        end;
        RetVal := FindNext( SRec );
      end;
      FindClose( SRec );
    end;
    Result := Result and (not Terminated);
  end; { Recurse }

begin
  { check parameters, set defaults if empty }
  if Path = ''
  then GetDir(0, FullPath)
  else FullPath := Path;
  if FullPath[Length(FullPath)] <> '\'
  then FullPath := FullPath + '\';
  if Mask = ''
  then Recurse( FullPath, '*.*' )
  else Recurse( FullPath, Mask );
end;

procedure THlcFileSearchThread.doSearchLogFunction;
begin
  try
    FResult := FSearchLogFunction(FPath, FSRec);
  except
    FResult := False;
  end;
end;

procedure THlcFileSearchThread.Execute;
var
  TheSearchRec: TSearchRec;
  iCount: integer;
begin
  if PathList.Count>0 then
  begin
    for iCount:=0 to PathList.Count-1 do
    begin
      if not Terminated then
      begin
        // Inform a dir change for each startpoint! TheSearchRec is a dummy
        InternalSearchLogFunction(PathList[iCount], TheSearchRec);
        FindRecursive(PathList[iCount], MaskList[iCount], InternalSearchLogFunction);
      end;
    end;
  end;
end;

function THlcFileSearchThread.InternalSearchLogFunction(
  const path: string; const SRec: TSearchRec): Boolean;
begin
  FPath := Path;
  FSRec := SRec;
  Synchronize(doSearchLogFunction);
  Result := FResult or Terminated;
end;

procedure THlcFileSearchThread.AddSearchTask(const SearchPath,
  SearchMask: string);
begin
  MaskList.Add(SearchMask);
  PathList.Add(SearchPath);
end;

destructor THlcFileSearchThread.Destroy;
begin
  MaskList.Free;
  MaskList:=nil;
  PathList.Free;
  PathList:=nil;
  inherited;
end;

end.
